@@ -18,9 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #if HAVE_STDLIB_H
 #include <stdlib.h>
@@ -37,13 +35,8 @@
 #include <ctype.h>
 #include <assert.h>
 
-#include "tdsodbc.h"
-
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
-
-TDS_RCSID(var, "$Id: native.c,v 1.29 2006-03-21 12:24:15 freddy77 Exp $");
+#include <freetds/odbc.h>
+#include <freetds/string.h>
 
 #define TDS_ISSPACE(c) isspace((unsigned char) (c))
 #define TDS_ISALPHA(c) isalpha((unsigned char) (c))
@@ -81,18 +74,19 @@ TDS_RCSID(var, "$Id: native.c,v 1.29 2006-03-21 12:24:15 freddy77 Exp $");
  * TRUNCATE -> ??
  */
 static SQLRETURN
-to_native(struct _hdbc *dbc, struct _hstmt *stmt, char *buf)
+to_native(struct _hdbc *dbc, struct _hstmt *stmt, DSTR *str)
 {
 	char *d, *s;
 	int nest_syntax = 0;
+	char *buf = tds_dstr_buf(str);
 
 	/* list of bit, used as stack, is call ? FIXME limites size... */
 	unsigned long is_calls = 0;
 	int server_scalar;
 
-	assert(dbc && buf);
+	assert(dbc);
 
-	server_scalar = TDS_IS_MSSQL(dbc->tds_socket) && dbc->tds_socket->product_version >= TDS_MS_VER(7, 0, 0);
+	server_scalar = TDS_IS_MSSQL(dbc->tds_socket) && dbc->tds_socket->conn->product_version >= TDS_MS_VER(7, 0, 0);
 
 	/*
 	 * we can do it because result string will be
@@ -122,7 +116,8 @@ to_native(struct _hdbc *dbc, struct _hstmt *stmt, char *buf)
 		if (*s == '{') {
 			char *pcall;
 
-			while (TDS_ISSPACE(*++s));
+			while (TDS_ISSPACE(*++s))
+				continue;
 			pcall = s;
 			/* FIXME if nest_syntax > 0 problems */
 			if (server_scalar && strncasecmp(pcall, "fn ", 3) == 0) {
@@ -131,9 +126,11 @@ to_native(struct _hdbc *dbc, struct _hstmt *stmt, char *buf)
 			}
 			if (*pcall == '?') {
 				/* skip spaces after ? */
-				while (TDS_ISSPACE(*++pcall));
+				while (TDS_ISSPACE(*++pcall))
+					continue;
 				if (*pcall == '=') {
-					while (TDS_ISSPACE(*++pcall));
+					while (TDS_ISSPACE(*++pcall))
+						continue;
 				} else {
 					/* avoid {?call ... syntax */
 					pcall = s;
@@ -178,7 +175,7 @@ to_native(struct _hdbc *dbc, struct _hstmt *stmt, char *buf)
 			*d++ = *s++;
 		}
 	}
-	*d = '\0';
+	tds_dstr_setlen(str, d - buf);
 	return SQL_SUCCESS;
 }
 
@@ -206,7 +203,7 @@ parse_const_param(const char *s, TDS_SERVER_TYPE *type)
 	if (isdigit(*s) || *s == '+' || *s == '-') {
 		errno = 0;
 		strtod(s, &end);
-		if (end != s && strcspn(s, ".eE") < (end-s)&& errno == 0) {
+		if (end != s && strcspn(s, ".eE") < (size_t) (end-s) && errno == 0) {
 			*type = SYBFLT8;
 			return end;
 		}
@@ -232,14 +229,15 @@ prepare_call(struct _hstmt * stmt)
 	SQLRETURN rc;
 	TDS_SERVER_TYPE type;
 
-	if (stmt->prepared_query)
-		buf = stmt->prepared_query;
-	else if (stmt->query)
-		buf = stmt->query;
-	else
+	if (tds_dstr_isempty(&stmt->query))
 		return SQL_ERROR;
 
-	if ((rc = to_native(stmt->dbc, stmt, buf)) != SQL_SUCCESS)
+	if ((!tds_dstr_isempty(&stmt->attr.qn_msgtext) || !tds_dstr_isempty(&stmt->attr.qn_options)) && !IS_TDS72_PLUS(stmt->dbc->tds_socket->conn)) {
+		odbc_errs_add(&stmt->errs, "HY000", "Feature is not supported by this server");
+		return SQL_SUCCESS_WITH_INFO;
+	}
+
+	if ((rc = to_native(stmt->dbc, stmt, &stmt->query)) != SQL_SUCCESS)
 		return rc;
 
 	/* now detect RPC */
@@ -247,7 +245,7 @@ prepare_call(struct _hstmt * stmt)
 		return SQL_SUCCESS;
 	stmt->prepared_query_is_rpc = 0;
 
-	s = buf;
+	s = buf = tds_dstr_buf(&stmt->query);
 	while (TDS_ISSPACE(*s))
 		++s;
 	if (strncasecmp(s, "exec", 4) == 0) {
@@ -274,7 +272,8 @@ prepare_call(struct _hstmt * stmt)
 	param_start = s;
 	--s;			/* trick, now s point to no blank */
 	for (;;) {
-		while (TDS_ISSPACE(*++s));
+		while (TDS_ISSPACE(*++s))
+			continue;
 		if (!*s)
 			break;
 		switch (*s) {
@@ -291,7 +290,8 @@ prepare_call(struct _hstmt * stmt)
 			--s;
 			break;
 		}
-		while (TDS_ISSPACE(*++s));
+		while (TDS_ISSPACE(*++s))
+			continue;
 		if (!*s)
 			break;
 		if (*s != ',') {
@@ -302,7 +302,9 @@ prepare_call(struct _hstmt * stmt)
 	stmt->prepared_query_is_rpc = 1;
 
 	/* remove unneeded exec */
-	memmove(buf, p, strlen(p) + 1);
+	s += strlen(s);
+	memmove(buf, p, s - p);
+	tds_dstr_setlen(&stmt->query, s - p);
 	stmt->prepared_pos = buf + (param_start - p);
 
 	return SQL_SUCCESS;
@@ -310,7 +312,7 @@ prepare_call(struct _hstmt * stmt)
 
 /* TODO handle output parameter and not terminated string */
 SQLRETURN
-native_sql(struct _hdbc * dbc, char *s)
+native_sql(struct _hdbc * dbc, DSTR *s)
 {
 	return to_native(dbc, NULL, s);
 }
